function [varX, varY, detFFast, eigenRatio] = CalculateCRLB(dx, dy, variance, isNoisy)
    % Compute the gradient and remove false gradient values
    %[dx, dy] = CalculateGradient(img1);
    %dx = dx(1:end-1,1:end-1);
    %dy = dy(1:end-1,1:end-1);
    dx = dx(:);
    dy = dy(:);
    % Compute the structure tensor
    N = length(dy);
    sdx2Fast = sum(dx.^2);
    sdy2Fast = sum(dy.^2);
    sdxdy = sum(dx.*dy);
    % If input image is noisy, try to remove the bias caused by the noise.
    if (isNoisy)
        sdx2Fast = sdx2Fast - 2*N*variance;
        sdy2Fast = sdy2Fast - 2*N*variance;
        sdxdy = sdxdy - N*variance;
    end
    A = [sdx2Fast sdxdy;sdxdy sdy2Fast];
    T = (sdx2Fast + sdy2Fast);
    detFFast = sdx2Fast*sdy2Fast - (sdxdy)^2;
    % Compute Eigenvalues of Structure Tensor for Eigenratio
    aux = sqrt((T^2/4)-detFFast);
    L1 = T/2 + aux;
    L2 = T/2 - aux;
    eigenRatio = L2/L1;
    
    % Compute the CRLB
    varX = ((variance * sdy2Fast) / detFFast);
    varY = ((variance * sdx2Fast) / detFFast);
end
